#!/usr/bin/perl -w

# $Id: add-scribe-groupe.pl,v 1.4 2004/09/09 09:46:27 joel Exp $
# Ajout du libellé du groupe créé par smbldap-groupadd.pl
# Enregistrement du partage associé au groupe si besoin
# Scribe 1.0
# Equipe Eole eole@ac-dijon.fr

use strict;
use FindBin;
use FindBin qw($RealBin);
use lib "$RealBin/";
use smbldap_tools;
use Net::LDAP;

use Getopt::Std;
my %Options;

my $ok = getopts('prs:t:n:m:?', \%Options);
if ( (!$ok) || (@ARGV < 1) || ($Options{'?'}) ) {
	print "Usage: $0 [-pstn?] groupname\n";
	print "  -p	avec partage RW\n";
	print "  -r	avec partage RO\n";
	print "  -s     nom du serveur (avec -p)\n";
	print "  -t	type du groupe (classe, section, groupe...)\n";
	print "  -n     niveau associé (pour Classe uniquement)\n";
	print "  -m     adresse mail de la liste de diffusion\n";
	print "  -?	show this help message\n";
	exit (1);
}

# nom du groupe
my $groupName = $ARGV[0];
# type du groupe
my $type = $Options{'t'};
if (!defined($type)) {
	print "Erreur option -t vide !";
	exit();
}
# niveau si classe
my $niveau = $Options{'n'};
my $mail = $Options{'m'};
my $groupsdn=$config{groupsdn};

# tableau des modifications
my @mods;

# spécifique aux groupes "Classe"
if (defined($niveau)) {
   push(@mods, 'ObjectClass' => ["classe","posixGroup","sambaGroupMapping"]);
   push(@mods, 'niveau' => "$niveau");
}   

# liste de diffusion
if (defined($mail)) {
   push(@mods, 'mail' => "$mail");
}

# convention actuelle : description = type + nom
push(@mods, 'description' => "$type $groupName");

# validation
my $ldap_master=connect_ldap_master();
my $modify = $ldap_master->modify ( "cn=$groupName,$groupsdn",
                                    'replace' => { @mods }
				  );
$modify->code && warn "failed to modify entry: ", $modify->error ;				  


# groupe pour le partage
my $pgroupe;

if ($Options{'p'}) {
   $pgroupe="sambaShareGroup";
}
if ($Options{'r'}) {
   $pgroupe="sambaShareGroupRO";
}

# si partage
if ($pgroupe) {
   # nom du serveur
   my $serveur = $Options{'s'};
   if (!defined($serveur)) {
	print "Erreur option -s vide !";
	exit();
   }
   # ldif d'un partage
   my $partagesdn="ou=local,ou=partages,".$config{etab}.",".$config{suffix};
   my $ldap_master=connect_ldap_master();
   my $add = $ldap_master->add("cn=smb://$serveur/$groupName,$partagesdn",
   	attrs => [
			'objectclass' => "sambaFileShare",
			'cn' => "smb://$serveur/$groupName",
			'sambaShareName' => "$groupName",
			'description' => "partage $groupName",
			'sambaShareDep' => "none",
			'sambaShareURI' => "\\\\$serveur\\$groupName",
			'sambaShareAdmin' => "Administrateurs",
			"$pgroupe" => "$groupName",
			'location' => "/",
			'sambaFilePAth' => "/home/workgroups/$groupName",
			'server' => "$serveur"
		]
	);
   $add->code && warn "Failed to add entry: ", $add->error;
   $ldap_master->unbind;
}
exit();
