#! /usr/bin/env python
# -*- coding: UTF-8 -*-

#########################################################
# utilitaire d'ajout d'un partage dans un annuaire LDAP
# licence GPL Eole
# Bruno Boiget <bruno.boiget@ac-dijon.fr>
#########################################################

from scribe.backend_conf import *
import manage_special_shares
import ldap,sys,getopt,os
from creole import parsedico

## ajout d'un partage samba pour un groupe
def add_share(sharename,groupe,filepath,drive):
    # on récupère la configuration eole
    dicoEole = parsedico.parse_dico()
    smb_serveur  = dicoEole["smb_netbios_name"]
    
    if filepath is None:
        filepath = "/home/workgroups/"+sharename
    
    # construction des données ldap
    dn='cn=smb://'+smb_serveur+'/'+sharename+',ou=local,ou=partages,ou='+dicoEole["numero_etab"]+',ou='+dicoEole["nom_academie"]+',ou=education,o=gouv,c=fr'
    
    modlistData=[("objectClass","sambaFileShare"),
    ( "cn","smb://"+smb_serveur+"/"+sharename),
    ("sambaShareName", sharename),
    ("description", sharename),
    ("sambaShareURI","\\\\"+smb_serveur+"\\"+sharename),
    ("sambaShareAdmin", "Administrateurs"),
    ("sambaShareGroup", groupe),
    ("location", "\\"),
    ("sambaFilePath", filepath),
    ("server", smb_serveur)]
    try:
        query=ldap.open(serveur)
        query.simple_bind_s("cn=admin, o=gouv, c=fr",ldap_passwd)
        result_lp=query.add_s(dn, modlistData)
        query.unbind()
    except:
        print "Erreur d'ajout du partage"
    else:
        if drive is not None:
            #mise à jour de special_shares
            manage_special_shares.add_special_share(sharename,drive)
        print "partage "+sharename+" créé"
    
def main():
    # définition du message d'aide
    help_msg ="""arguments:
         -n (--nom) nom du partage
         -g (--groupe) groupe auquel appartient le partage 
         -c (--chemin) chemin du répertoire à partager sur le serveur
            optionnel : /home/workgroups/nom_du_partage par défaut
         -l (--lettre) lettre sur laquelle connecter le partage sous windows
            optionnel : choisi par windows par défaut
         -h (--help) (ce message)

     exemple : ./ajout_partage.py -n cd -g groupe_cd -c /mnt/cdrom -l J:
     """

    # récupération des arguments de la ligne de commande
    try:
        options, args = getopt.getopt(sys.argv[1:],'n:g:c:l:h',
                                 ['nom=',
                                  'groupe=',
                                  'chemin=',
                                  'lettre=',
                                  'help'])
    except getopt.GetoptError:
        # on récupère les information de l'exception
        er_type, er_value, er_traceback = sys.exc_info()
        er_msg = str(er_value)
        # on prévient l'utilisateur que l'option nécessite un argument
        sys.exit("l'option -%s est inconnue ou nécessite un argument" % er_msg[er_msg.index('-')+1])

    name = None
    groupe = None
    path = None
    letter = None

    for (opt,val) in options:
        if (opt == '-n') or (opt == '--nom'):
            name = val

        elif (opt == '-g') or (opt == '--groupe'):
            groupe = val

        elif (opt == '-c') or (opt == '--chemin'):
            path = val

        elif (opt == '-l') or (opt == '--lettre'):
            letter = val

        elif (opt == '-h') or (opt == '--help'):
            sys.exit(help_msg)

    # vérification des arguments:
    # arguments obligatoires
    if name is None:
        sys.exit("\nerreur, nom de partage est obligatoire\n"+help_msg)
    if groupe is None:
        sys.exit("\nerreur, le groupe d'appartenance est obligatoire\n"+help_msg)
    # arguments optionnels    
    if path is not None:
        # on vérifie l'existence du répertoire
        if not os.path.isdir(path):
            print("\nattention, le chemin spécifié n'a pas été trouvé !")

    # ajout du partage
    add_share(name,groupe,path,letter)

if __name__ == '__main__':
    main()
