#!/usr/bin/env python
# -*- coding: UTF-8 -*-

#########################################################
# utilitaire d'attribution des modeles Windows pour Scribe
# licence GPL Eole
# Joël Cuissinat <joel.cuissinat@ac-dijon.fr>
#########################################################

###################
# modules importés
###################
import sys
import ldap
import getopt
import ldif
from os import system
from scribe.backend_conf import *
#from creole import parsedico

#####################
# variables globales
#####################
passwd = ldap_passwd


## liste des gens du groupe 
def get_users(passwd,serveur,groupe):
    try:
        query=ldap.open(serveur)
        query.simple_bind_s("cn=admin,o=gouv,c=fr", passwd)
        result_user=query.search_s("o=gouv, c=fr", ldap.SCOPE_SUBTREE, \
               "(&(objectclass=posixGroup)(cn="+groupe+"))")
        query.unbind()	
    except:
        result_user=[]
    nb=0
    uids=result_user[0][1]["memberUid"]
    users=[]
    for uid in uids:
        try:	    
            users.append(get_dn(passwd,serveur,uid))
            nb+=1
	except:
	    pass
    return (users,nb)


## retourne le modele appliqué à une personne
def get_modele(passwd,serveur,uid):
    try:
        query=ldap.open(serveur)
        query.simple_bind_s("cn=admin,o=gouv,c=fr", passwd)
        r=query.search_s("o=gouv, c=fr", ldap.SCOPE_SUBTREE, \
               "(&(objectclass=sambaSAMAccount)(uid="+uid+"))")
        query.unbind()
	return r[0][1]['employeeType'][0]
    except:
        return ""
    
## retourne le dn associé à l'uid
def get_dn(passwd,serveur,uid):
    try:
        query=ldap.open(serveur)
        query.simple_bind_s("cn=admin,o=gouv,c=fr", passwd)
        r=query.search_s("o=gouv, c=fr", ldap.SCOPE_SUBTREE, \
               "(&(objectclass=sambaSAMAccount)(uid="+uid+"))")
        query.unbind()
	return r[0][0]
    except:
        return ""
    
## attribution du modele aux utilisateurs
def set_modele (passwd,serveur,utilisateurs,modele):
    ldif=""
    for dn in utilisateurs:
        ldif+="dn: "+dn+"\n"  
        ldif+="changetype: modify\n"
	ldif+="replace: employeetype\n"
	ldif+="employeetype: "+modele+"\n\n"

    cmd = "echo \""+ldif+"\" | ldapadd -x -D cn=admin,o=gouv,c=fr -w "+ldap_passwd +" > /dev/null"
    r = system(cmd)
    if r != 0 :
        if r==17408 :
            print "erreur d'ajout ldif"
        elif r==5120 :
            print "erreur de modification ldif"
        else :
            print "erreur inconnue sur lors de la modification LDAP"
    else:
	return 1

## suppression de modèles aux utilisateurs
def supp_modele (passwd,serveur,utilisateurs):
    ldif=""
    for dn in utilisateurs:
        ldif+="dn: "+dn+"\n"  
        ldif+="changetype: modify\n"
	ldif+="delete: employeetype\n\n"

    cmd = "echo \""+ldif+"\" | ldapadd -x -D cn=admin,o=gouv,c=fr -w "+ldap_passwd +" > /dev/null 2>&1 "
    r = system(cmd)
    if r != 0 :
        if r==17408 :
            print "erreur d'ajout ldif"
        elif r==5120 :
            print "erreur de modification ldif"
        else :
            print "erreur inconnue sur lors de la modification LDAP"
    else:
	print 1


## message d'aide
def help():
    sys.exit("""   
    options possibles :
	       -u (--uid)    U : uid de l'utilisateur
	       -m (--modele) M : nom modèle à attribuer
	       -g (--groupe) G : nom du groupe
	       -s (--suppr)  U : suppression du modèle pour cet uid
	       -h (--help)     : affiche ce message

    Exemples :
	    modèle utilisé pour un utilisateur      : attribue-modele.py -u utilisateur
	    attribuer un modèle à un utilisateur    : attribue-modele.py -u utilisateur -m modele
	    attribuer un modèle à un groupe         : attribue-modele.py -g groupe -m modele	    
    """)


# programme principal
def main():
    # gestion des options
    uid=""
    modele=""
    groupe=""
    suppr=""

    try:
	    options, args = getopt.getopt(sys.argv[1:],'u:g:m:s:h',
            ['uid=',
	     'groupe=',
	     'modele=',
	     'suppr=',
             'help'])
             
    except getopt.GetoptError:
        er_type, er_value, er_traceback = sys.exc_info()
        er_msg = str(er_value)
        sys.exit("l'option -%s est inconnue ou nécessite un argument" % er_msg[er_msg.index('-')+1])

    for (opt,val) in options:
        # uid
        if (opt == '-u') or (opt == '--uid'):
            uid = val
        # modele
        if (opt == '-m') or (opt == '--modele'):
            modele = val
        # pour un groupe
        if (opt == '-g') or (opt == '--groupe'):
            groupe=val
        # suppression
        elif (opt == '-s') or (opt == 'suppr'):
	    suppr=val
        # et l'aide...
        elif (opt == '-h') or (opt == '--help'):
	    help()	   
    
    # afficher le modèle d'un utilisateur
    if uid != "" and modele == "":
        print get_modele(passwd,serveur,uid)

    # attribuer un modèle à un utilisateur
    elif uid != "" and modele != "":
	dn=get_dn(passwd,serveur,uid)
	if dn != "":
            print set_modele(passwd,serveur,[dn],modele)
        else:
	    print 0
	    
    # attribuer un modèle aux membres d'un groupe
    elif groupe != "" and modele != "":
	users,nb=get_users(passwd,serveur,groupe)
	if nb>0 :
	    set_modele(passwd,serveur,users,modele)
        print nb

    # suppression d'un modèle
    elif suppr != "" :
        dn=get_dn(passwd,serveur,suppr)
        if dn != "":
	    supp_modele(passwd,serveur,[dn])
	else:
	    print 0
	    
    # aucune option
    else :
	help()


if __name__ == "__main__":
    main()

