#!/bin/bash
#
# utilitaire de gestion de l'application @SSR
#
# Klaas TJEBBES
#
#

APACHE_CONF='/etc/apache2/sites-enabled/80_assr.conf'
ASSR_DIR='/home/serveurassr/'
ASSR_COPIES_DIR='/home/workgroups/commun/logiciels/assr/copies/'
ADMIN_LINK='/home/workgroups/commun/logiciels/assr/serveurassr'

F_URL_ASSR='/var/www/ead/config/url_assr'
F_PASS_ASSR='/var/www/ead/config/pass_assr'

CREATE_USER_SCRIPT='/usr/share/eole/backend/assr_user_create.py'

AT_ASSR='/var/www/ead/config/at_assr'

## ACTIVATION
genconf(){
    cat > $APACHE_CONF << EOF
#
# Configuration pour @SSR
#
 
Alias /assr /home/serveurassr
Alias /serveurassr /home/serveurassr

<Directory /home/serveurassr>
    AddDefaultCharset ISO-8859-1
    Options FollowSymLinks
    AllowOverride None
    order allow,deny
    allow from all
</Directory>
EOF
}

create_assr(){
    # Création du répertoire d'install
    [ ! -d $ASSR_DIR ] && mkdir -p $ASSR_DIR
    # ajout du répertoire pour les copies
    [ ! -d $ASSR_COPIES_DIR ] && mkdir -p $ASSR_COPIES_DIR
    # lien dans commun\logiciels\assr
    [ ! -L $ADMIN_LINK ] && ln -s $ASSR_DIR $ADMIN_LINK
    # les droits sur le répertoire d'installation
    chmod 750 $ASSR_DIR
    # droits www-data
    setfacl -Rm u:www-data:rwx $ASSR_DIR 2>/dev/null
    setfacl -dRm u:www-data:rwx $ASSR_DIR 2>/dev/null
    setfacl -Rm g:www-data:r-x $ASSR_DIR 2>/dev/null
    setfacl -dRm g:www-data:r-x $ASSR_DIR 2>/dev/null
    # sur le répertoire copies
    setfacl -Rm g:professeurs:rwx $ASSR_COPIES_DIR 2>/dev/null
    setfacl -dRm g:professeurs:rwx $ASSR_COPIES_DIR 2>/dev/null
    # élèves, pas le droit de lister le contenu du répertoire copies
    setfacl -m g:eleves:-wx $ASSR_COPIES_DIR 2>/dev/null
    # droit de lire les copies qui sont dedans
    setfacl -dm g:eleves:rwx $ASSR_COPIES_DIR 2>/dev/null
    $CREATE_USER_SCRIPT $1 $2 $3
}

# SUPPRESSION
del_copies(){
    find $ASSR_COPIES_DIR -type f -mtime +122 -exec rm -f {} \;
}

del_assr(){
    rm -rf $ASSR_DIR
    rm -f $ADMIN_LINK
}

del_conf(){
    rm -f $APACHE_CONF
}

# DIVERS
apache_restart(){
    echo "service apache2 restart" | at now &> /dev/null
}

#Fonctions à lancer
active(){
    genconf
    create_assr $1 $2 $3
    apache_restart
}

desactive(){
    del_conf
    apache_restart
}

supprime(){
    del_conf
    del_copies
    del_assr
    apache_restart
}

telecharge(){
    if [ -z $3 ]
    then
	echo "Usage : "`basename $0`" telecharge SERVEUR_RSYNC USER PASS"
	exit 1
    fi
    serv_assr=$1
    user=$2
    pass=$3
    echo $serv_assr > $F_URL_ASSR
    echo $user >> $F_URL_ASSR
    echo $pass > $F_PASS_ASSR
    chmod 644 $F_PASS_ASSR
    HMAJ=$[( $RANDOM % 3) +20]
    MMAJ=$[( $RANDOM % 60)]
    if [ $MMAJ -lt 10 ]
    then
            MMAJ="0"$MMAJ
    fi
#    HMAJ=17
#    MMAJ=15
    echo "Le téléchargement sera lancé à $HMAJ:$MMAJ" > $AT_ASSR
    echo /usr/share/eole/backend/dl_assr.sh | at $HMAJ:$MMAJ
}

annul_telech(){
    heuref=`cat $AT_ASSR |awk '{print $6}'`
    atq | while read line
    do
	echo $line | grep -q $heuref
	if [ $? -eq 0 ]
	then
	    id=`echo $line|awk '{print $1}'`
	    atrm $id
	fi
    done
    rm -f $AT_ASSR
}

case $1 in
    active)
	active $2 $3 $4
	;;
    desactive)
	desactive
	;;
    supprime)
    	supprime
        ;;
    telecharge)
        telecharge $2 $3 $4
	;;
    annul_telech)
        annul_telech
	;;
    *)
        echo "Usage : "`basename $0`" active|desactive|supprime|telecharge|annul_telech"
	exit 1
esac
