#!/usr/bin/env python
# -*- coding: UTF-8 -*-

#########################################################
# Appel des différents scripts pour ajouter un élève
# licence GPL Eole
# Joël Cuissinat <joel.cuissinat@ac-dijon.fr>
#########################################################

###################
# modules importés
import sys
import getopt
from os import system
from creole import parsedico
###################

#####################
# variables globales
#skel_rep = "/usr/share/eole/backend/skel"
tool_path= "/usr/share/eole/backend"
# utilisation du dictionnaire EOLE
dicoEole = parsedico.parse_dico()
samba_server = dicoEole["smb_netbios_name"]
rne         = dicoEole["numero_etab"]
dom         = dicoEole["domaine_messagerie_etab"]
#####################

def main():
    profil = '0'
    # gestion des options
    try:
        options, args = getopt.getopt(sys.argv[1:],'u:c:n:m:p:f:x:d:o:D:q:P:h',
            ['user=',
             'classe=',
             'niveau=',
             'mdp=',
             'prenom=',
             'fnom=',
             'sexe=',
             'date_naiss=',
             'numero=',
             'domain=',
             'quota=',
             'profil=',
             'help'])
             
    except getopt.GetoptError:
        er_type, er_value, er_traceback = sys.exc_info()
        er_msg = str(er_value)
        sys.exit("l'option -%s est inconnue ou nécessite un argument" % er_msg[er_msg.index('-')+1])

    for (opt,val) in options:
        # nom de l'élève
        if (opt == '-u') or (opt == '--user'):
            user = val
        # classe
        elif (opt == '-c') or (opt == '--classe'):
            classe = val
        # niveau
        elif (opt == '-n') or (opt == '--niveau'):
            niveau = val
        # mot de passe
        elif (opt == '-m') or (opt == '--mdp'):
            password = val
        # prénom
        elif (opt == '-p') or (opt == '--prenom'):
            prenom = val
        # nom (de famille)
        elif (opt == '-f') or (opt == '--fnom'):
            nom = val
        # sexe 1/0
        elif (opt == '-x') or (opt == '--sexe'):
            sexe = val
        # date de naissance
        elif (opt == '-d') or (opt == '--date_naiss'):
            date_naiss = val
        # numero élève
        elif (opt == '-o') or (opt == '--numero'):
            numero = val
        # domaine
        elif (opt == '-D') or (opt == '--domain'):
            if val=="restreint" :
                domaine = "i-"+dom
            else:
                domaine = dom
        # quota
        elif (opt == '-q') or (opt == '--quota'):
            quota = val
        # profil
        elif (opt == '-P') or (opt == '--profil'):
            profil = val

        # et... l'aide 
        elif (opt == '-h') or (opt == '--help'):
            sys.exit("""options :
                -u login eleve
                -c classe
                -n niveau
                -m mot de passe
                -p prenom
                -f nom de famille
                -x sexe (1=M., 2=Mme, 3=Mlle)
                -d date de naissance (aaaammjj)
                -o numero élève
                -D domaine mail (restreint | internet)
                -q quota
                -P profil (1=local, 2=obligatoire-1, 3=obligatoire-2)
                -h (aide)
                """)
    
    # definition des scripts
    script1="/usr/sbin/smbldap-useradd"
    script2=tool_path+"/passwd-eole.pl"
    script3=tool_path+"/add-scribe-eleve.pl"
    script5="/usr/share/ead/rep-eleve.sh"
    
    # definition de leurs paramètres
    para1 = "-a -A 1 " 
    #para1+= "-m -k "+skel_rep+" " 
    para1+= "-o \"ou=local,ou=eleves\" "
    para1+= "-s /bin/false "
    para1+= "-d /home/"+user[0]+"/"+user+" "
    para1+= "-D U: -H U "
    para1+= "-C \\\\\\\\"+samba_server+"\\\\"+user+"\\\\perso "
    if profil == '2':
        para1+=" -F \\\\\\\\"+samba_server+"\\\\netlogon\\\\profil "
    elif profil == '3':
        para1+=" -F \\\\\\\\"+samba_server+"\\\\netlogon\\\\profil2 "
    else:
        para1+= "-F \"\" "
    #para1+= "-F \\\\\\\\"+samba_server+"\\\\profiles\\\\"+user+" "
    #para1+= "-E "+user+".bat "
    para1+= "-g eleves "
    para1+= "-G "+classe+","+niveau+",domainUsers "+user
    
    para2 = "-u "+user+" -p "+password

    para3 = "-p \""+prenom+"\" "
    para3+= "-n \""+nom+"\" "
    para3+= "-c "+sexe+" "
    para3+= "-b "+date_naiss+" "
    para3+= "-e "+numero+" "
    para3+= "-d "+domaine+" "
    para3+= "-M "+niveau+" -D "+classe+" "
    para3+= "-R "+rne+" "+user
    
    para5 = user+" "+classe+" "+quota
    
    #print script1+" "+para1
    #execution
    retour=system (script1+" "+para1)
    if retour != 0: 
        sys.exit("""Erreur d'ajout de l'utilisateur Samba""")
        
    retour=system (script2+" "+para2)
    #print  script2+" "+para2
    if retour != 0: 
        sys.exit("""Erreur lors du changement de mot de passe""")

    #print script3+" "+para3
    retour=system (script3+" "+para3)
    if retour != 0: 
        sys.exit("""Erreur lors de la modification de l'utilisateur""")
    
    retour=system (script5+" "+para5)
    
    entete="Bienvenue sur "+domaine
    message="Votre compte de messagerie est actif"
    mail=user+"@"+domaine
    retour=system ("echo \""+message+"\" | mail "+mail+" -s \""+entete+"\"");
    
    formule="L'utilisateur "+prenom+" "+nom+" a été ajouté"
    sys.exit(formule)


if __name__ == "__main__":
    main()
