#!/usr/bin/env python
# -*- coding: UTF-8 -*-

#########################################################
# Appel des différents scripts pour ajouter un élève
# licence GPL Eole
# Joël Cuissinat <joel.cuissinat@ac-dijon.fr>
#########################################################

###################
# modules importés
import sys
import getopt
from os import system
from creole import parsedico
###################

#####################
# variables globales
tool_path = "/usr/share/eole/backend"
# utilisation du dictionnaire
dicoEole = parsedico.parse_dico()
samba_server = dicoEole["smb_netbios_name"]
rne          = dicoEole["numero_etab"]
domaine      = dicoEole["domaine_messagerie_etab"]
#####################

def main():

    groupes = ""
    profil = "0"
    date_naiss = ""
    # gestion des options
    try:
        options, args = getopt.getopt(sys.argv[1:], 'u:c:m:p:f:d:M:w:x:z:q:G:P:h',
            ['user=',
             'classe=',
             'mdp=',
             'prenom=',
             'fnom=',
             'sexe=',
             'date_naiss=',
             'mail=',
             'quota=',
             'forward=',
             'droits=',
             'groupes=',
             'profil=',
             'help'])
             
    except getopt.GetoptError:
        er_type, er_value, er_traceback = sys.exc_info()
        er_msg = str(er_value)
        sys.exit("l'option -%s est inconnue ou nécessite un argument" % er_msg[er_msg.index('-')+1])

    for (opt, val) in options:
        # login
        if (opt == '-u') or (opt == '--user'):
            user = val
        # classe
        elif (opt == '-c') or (opt == '--classe'):
            classe = val
        # mot de passe
        elif (opt == '-m') or (opt == '--mdp'):
            password = val
        # prénom
        elif (opt == '-p') or (opt == '--prenom'):
            prenom = val
        # nom (de famille)
        elif (opt == '-f') or (opt == '--fnom'):
            nom = val
        # civilité
        elif (opt == '-x') or (opt == '--sexe'):
            sexe = val
        # date de naissance
        elif (opt == '-d') or (opt == '--date_naiss'):
            date_naiss = val
        # domaine
        elif (opt == '-M') or (opt == '--mail'):
            mail = val
        # niveau de droits
        elif (opt == '-z') or (opt == '--droits'):
            droits = val
        # groupes
        elif (opt == '-G') or (opt == '--groupes'):
            groupes = val
        # quota
        elif (opt == '-q') or (opt == '--quota'):
            quota = val
        # profil
        elif (opt == '-P') or (opt == '--profil'):
            profil = val
            
        # et... l'aide 
        elif (opt == '-h') or (opt == '--help'):
            sys.exit("""options possibles :
                -u login prof
                -c classe ("autre" si droit=0, $classe si droit=2)
                [-G groupe1,groupe2]
                -m mot de passe
                -p prenom
                -f nom de famille
                -x sexe (1=M., 2=Mme, 3=Mlle)
                -d date de naissance (aaaammjj)
                -M adresse mail ou mot clé "local"
                -z niveau de droits
                -q quota
                -P profile (1=local, 2=obligatoire-1, 3=obligatoire-2)
                -h (aide)
                """)

    # definition des scripts
    script1 = "/usr/sbin/smbldap-useradd"
    script2 = tool_path+"/passwd-eole.pl"
    script3 = tool_path+"/add-scribe-admin.pl"
    #script4="/usr/share/ead/set_quota.py"
    script5 = "/usr/share/ead/rep-prof.sh"

    # definition de leurs paramètres
    para1 = "-a -A 1 " 
    para1 += "-o \"ou=local,ou=personnels\" "
    #para1 += "-km "+skel_rep+" " 
    para1 += "-s /bin/false "
    para1 += "-d /home/"+user[0]+"/"+user+" "
    para1 += "-D U: -H U "
    para1 += "-C \\\\\\\\"+samba_server+"\\\\"+user+"\\\\perso "
    if profil == '2':
        para1 += " -F \\\\\\\\"+samba_server+"\\\\netlogon\\\\profil "
    elif profil == '3':
        para1 += " -F \\\\\\\\"+samba_server+"\\\\netlogon\\\\profil2 "
    else:
        para1 += "-F \"\" "
    #para1 += "-F \\\\\\\\"+samba_server+"\\\\profiles\\\\"+user+" "
    #para1 += "-E "+user+".bat "
    para1 += "-g professeurs "
    if groupes != "" :
        para1 += "-G "+groupes+",domainUsers "
    else :
        para1 += "-G domainUsers "
    para1 += " "
    para1 += user
    
    para2 = "-u "+user+" -p "+password

    para3 = "-p \""+prenom+"\" "
    para3 += "-n \""+nom+"\" "
    para3 += "-c \""+sexe+"\" "
    if date_naiss != '':
        para3 += "-b \""+date_naiss+"\" "
    if mail == 'local' :
        para3 += "-d "+domaine+" "
        # pour le code qui suit
        mail = user+"@"+domaine
    else:
        para3 += "-m "+mail+" "
    para3 += "-D "+classe+" "
    para3 += "-M autre " # à virer !
    para3 += "-a "+droits+" "+user
    
    #para4 = user+" "+quota

    para5 = user+" "+quota

    #execution
    retour = system (script1 + " " + para1)
    if retour != 0: 
        sys.exit("""Erreur d'ajout de l'utilisateur Samba""")
        
    retour = system ("echo -e \""+password+"\n"+password+"\n\" | " + script2 + " " + para2)
    if retour != 0: 
        sys.exit("""Erreur lors du changement de mot de passe""")

    retour = system (script3 + " " + para3)
    if retour != 0: 
        sys.exit("""Erreur lors de la modification de l'utilisateur""")

    #retour=system (script4+" "+para4)

    retour = system (script5 + " " + para5)
    
    entete = "Bienvenue sur Scribe"
    message = "Votre compte "+user+" sur le serveur Scribe est actif"
    retour = system ("echo \""+message+"\" | mail "+mail+" -s \""+entete+"\"");

    formule = "L'utilisateur "+prenom+" "+nom+" a été ajouté"
    sys.exit(formule)


if __name__ == "__main__":
    main()
