#! /usr/bin/env python
# -*- coding: UTF-8 -*-
 
#########################################################
# utilitaire de generation de fichiers netlogon depuis LDAP
# licence GPL Eole
# Bruno Boiget <bruno.boiget@ac-dijon.fr>
# Joel Cuissinat <joel.cuissinat@ac-dijon.fr>
#########################################################

import sys, os, time, ldap, getopt

import commands

from special_shares import *
from scribe.backend_conf import *
from scribe import backend

from creole import parsedico


class UserUnknown(Exception):
    pass



####################################
## Fonctions de consultation LDAP ##
####################################

class Ldap:
    def __init__(self, serveur='127.0.0.1'):
        self.serveur = serveur
        self.__passwd = ldap_passwd
        # connexion LDAP unique pour toutes les requètes
        self.query = ldap.open(self.serveur)
        self.query.simple_bind_s('cn=admin,o=gouv,c=fr', self.__passwd)
        self.dicoEole = parsedico.parse_dico()

    def unbind(self):
        """Fermeture de la connexion LDAP
        """
        self.query.unbind()

    ## récupération du chemin du répertoire perso d'un utilisateur
    def get_user_homedrive(self, user):
        """renvoie la lettre de lecteur pour le dossier perso
        """
        try:
            result_user = self.query.search_s('o=gouv,c=fr', ldap.SCOPE_SUBTREE, \
                '(&(objectclass=sambaSAMAccount)(objectclass=inetOrgPerson)(uid=%s))'%(user))
            # création de la liste des noms récupérés
            return (str(result_user[0][1]['sambaHomeDrive'][0]), '\\\\%s\\PERSO'%(self.dicoEole['smb_netbios_name']))
        except:
            raise UserUnknown
    
    def get_shares(self):
        """liste de tous les répertoires partagés
        renvoie le tuple (chemin UNC, groupe associé au partage, nom du partage)
        """
        try:
            result_share = self.query.search_s('o=gouv,c=fr', ldap.SCOPE_SUBTREE, '(objectclass=sambaFileShare)')
        except:
            result_share = []
    
        list_share = []
        for share in result_share:
            list_share.append((share[1]['sambaShareURI'][0], share[1]['sambaShareGroup'][0], share[1]['sambaShareName'][0]))
        
        return list_share
    
    
    ## liste des groupes associés (ou non) à un utilisateur
    def get_usergroups(self, user, gtype=None):
        """renvoie soit la liste des groupes disponibles (gtype = 'dispo') (ceux où n'est pas inscrit 'user')
        soit la liste des groupes de 'user'
        """
        try:
            result_dispo = self.query.search_s('o=gouv,c=fr', ldap.SCOPE_SUBTREE, '(objectclass=posixGroup)')
        except:
            result_dispo = []
            
        liste_group = []
        
        for group in result_dispo:
            try: gname = group[1]['cn'][0]
            except: continue
            
            try: members = group[1]['memberUid']
            except: members = []
    
            if gtype == 'dispo':
                if user not in members:
                    liste_group.append(gname)
            elif gtype == 'tous':
                liste_group.append(gname)
            else:
                if user in members:
                    liste_group.append(gname)
    
        liste_group.sort()
        return liste_group
    
    
    def get_user_shares(self, user):
        """ fonction de récupération des partages associés à un utilisateur
        user : l'utilisateur en question
        """
        # recherche des partages associés à ces groupes
        # liste de tous les partages et leur groupe associé
        shares = self.get_shares()
        groups = self.get_usergroups(user)
    
        # construction de la liste des  partages de l'utilisateur
        # -> liste de tuples de type (nom,chemin)
        # le nom sert à identifier des cas spéciaux (applidos et minedu)
        share_user = []
        for share in shares:
            sharepath, sharegroup, sharename = share
            if sharegroup in groups:
                share_user.append((sharename, sharepath))
    
        return share_user


#################################
## Création du fichier bat
##################################
class logon:
    def __init__(self, serveur='127.0.0.1'):
        self.serveur = serveur
        self.ldap = Ldap(self.serveur)

        # on récupère le nom du serveur samba
        Dico = parsedico.parse_dico()
        try: self.smb_serveur = Dico['smb_netbios_name']
        except: sys.exit("Erreur, le nom de ce serveur n'est pas défini dans la configuration Eole")

    def build_netlogon(self, user, os_type, machine, adresse_ip):
        """ fonction de construction des scripts de connexion windows
        user : utilisateur, os_type : type de l'OS selon Samba, machine : nom de la machine, adresse_ip
        le fichier est du format .bat pour Win95 et .txt pour le reste
        """
        # suppression d'éventuels espaces
        self.user = user.strip().replace(' ','')
        self.os_type = os_type.strip().replace(' ','')
        self.machine = machine.strip().replace(' ','')
        self.adresse_ip = adresse_ip.strip().replace(' ','')

        self.groups = self.ldap.get_usergroups(self.user)
        self.user_shares = self.ldap.get_user_shares(self.user)

        # le répertoire de l'utilisateur sans espaces
        self.user_dir = os.path.join('/home', self.user[0], self.user)
        # présentation groupee d'office sur Scribe
        self.dir_groupes = os.path.join(self.user_dir, 'groupes')
        
        try:
            self.homedrive, self.homepath = self.ldap.get_user_homedrive(self.user)
        except UserUnknown:
            sys.exit("erreur : utilisateur non défini")

        # Ouverture de session :
        # Enregistrement de la connexion
        self.log_connexion()
        #   création de /home/u/user/.ftp/
        #   création de /home/u/user/groupes/
        backend.gen_ftpdir(self.user)
        self.gen_groupedir()
        # Génération du fichier de logon lu par le client
        self.gen_fich()
        # Fermeture de la connexion LDAP
        self.ldap.unbind()

    def log_connexion(self):
        """Enregistrement de la connexion
        """
        try: primgrp = commands.getoutput('id -gn %s'%self.user)
        except: primgrp = None
        try:
            sdate = commands.getoutput('date +%a" "%d" "%b" "%Y" "%H":"%M').capitalize()
            cmd = 'echo CONNECTION %s %s %s %s %s %s'%(sdate, self.user, primgrp, self.machine, self.os_type, self.adresse_ip)
            os.system('%s >> /var/log/samba/%s.log'%(cmd, self.machine))
            os.system('%s >> /var/log/samba/connexions.log'%(cmd))
        except:
            pass

    def gen_groupedir(self):
        """Suppression du répertoire contenant les liens vers les groupes de l'utilisateur
        et (re)construction du répertoire
        """
        # visiblement os.removedirs()  n'est pas si récursif que ça !
        os.system('rm -rf %s'%self.dir_groupes)
        os.makedirs(self.dir_groupes)
        os.system('/bin/chown %s %s'%(self.user, self.dir_groupes))
        os.system('setfacl -bk %s'%self.dir_groupes)
        os.chmod(self.dir_groupes,0500) # pas d'écriture à la racine  S:

    def gen_fich(self):
        """Génération du fichier lu par le client
        """
        serv_path = '\\\\%s\\'%self.smb_serveur
        
        # Gestion des partages
        self.milieu_script = ''
        special_shares.update(eole_shares)
        for share in self.user_shares:
            name, path = share
            if name in special_shares.keys(): # soit une lettre de lecteur dédiée
                self.milieu_script += self.gen_lecteur_bloc(special_shares[name], path)
            else: # soit un lien dans le répertoire "groupes"
                os.symlink(os.path.join('/home/workgroups/', name), os.path.join(self.dir_groupes, name))
        # Connexion du répertoire personnel (lettre de lecteur définie dans LDAP)
        if self.os_type in ['Win95', 'Samba']:
            # cette connexion est automatique sous XP et 2K (NT?)
            self.milieu_script += self.gen_lecteur_bloc(self.homedrive, self.homepath)

        if self.os_type == 'Win95': self.milieu_script += self.gest_win95()
        else: self.milieu_script += self.gen_esu()
        # Les scripts additionnels
        self.debut_script, self.fin_script = self.get_scripts()
        # MAINTENANT ECRIRE LE FICHIER
        self.write_fich()
    
    def gen_lecteur_bloc(self, lettre, share):
        """Génère un bloc de ligne pour le montage d'un lecteur réseau "share" sur la lettre "lettre"
        ou sur * sinon
        """
        lettre = lettre[:1] # juste la lettre, sans les ":"
        while share[-1] == '\\': share = share[:-1] # le partage sans \ à la fin
        sharename = share.split('\\')[-1].replace('$', '')
        if self.os_type == 'Win95':
            # on définit la commande net use selon le type d'os (NT ou 9X)
            label = '%s%s'%(lettre, sharename)
            chaine_net_use = 'NET USE '
            bloc = self.gen_cmd_line('IF EXIST %s:\\nul goto err%s'%(lettre, label))
            bloc += self.gen_cmd_line('%s %s: %s'%(chaine_net_use, lettre, share))
            bloc += self.gen_cmd_line('goto suit%s'%(label))
            bloc += self.gen_cmd_line(':err%s'%(label))
            bloc += self.gen_cmd_line('%s * %s'%(chaine_net_use, share))
            bloc += self.gen_cmd_line(':suit%s'%(label))
        else:
            bloc = 'lecteur,%s:,%s\n'%(lettre, share)
        return bloc

    def gen_cmd_line(self, line, hide=False, nowait=False):
        """génère une ligne DOS (avec le bon retour chariot)
        """
        if self.os_type == 'Win95':
            saut_ligne = chr(13)+chr(10)
            return '%s%s'%(line, saut_ligne)
        else:
            chaine = 'cmd,%s'
            if hide:
                chaine += ',HIDDEN'
            if nowait:
                chaine += ',NOWAIT'
            chaine += '\n'
            return chaine%line
        return ''

    def get_scripts(self):
        """Gestion des scripts externes
        """
        # motif à rechercher dans les fichiers externes (debut_script %motif% fin_script)
        motif_include = [ "%%NetUse%%", "%NetUse%" ]
        # chemin par défaut des scripts externes
        buffer_debut,  buffer_fin = '', ''
        # on traite ceux qui sont présents
        for chemin in self.get_scripts_list():
            if os.path.isfile(chemin):
                debut = 1
                for line in file(chemin,"r").read().splitlines():
                    if line.strip() in motif_include:
                        debut = 0
                    else:
                        if debut == 1: buffer_debut += self.parse_line(line.strip())
                        else: buffer_fin += self.parse_line(line.strip())
        return (buffer_debut, buffer_fin)

    def get_scripts_list(self):
        """On créé la liste des fichiers possibles
        +--path_scripts/users/<user>.txt
        +--path_scripts/groups/<group>.txt
        +--path_scripts/machines/<machine>.txt
        +--path_scripts/os/<os>.txt
        +--path_scripts/os/<os>/<group>.txt
        +--path_scripts/os/<os>/<user>.txt
        """
        path_scripts = '/home/netlogon/scripts'
        scripts_ext = '.txt'
        chemins = [os.path.join(path_scripts, 'users', '%s%s'%(self.user, scripts_ext))]
        chemins.append(os.path.join(path_scripts, 'machines', '%s%s'%(self.machine, scripts_ext)))
        for group in self.groups:
            chemins.append(os.path.join(path_scripts, 'groups', '%s%s'%(group, scripts_ext)))
            chemins.append(os.path.join(path_scripts, 'os', '%s'%self.os_type, '%s%s'%(group, scripts_ext)))
        chemins.append(os.path.join(path_scripts, 'os', '%s%s'%(self.os_type, scripts_ext)))
        chemins.append(os.path.join(path_scripts, 'os', self.os_type, '%s%s'%(self.user, scripts_ext)))
        return chemins

    def parse_line(self, line):
        items = [ i.strip() for i in line.split(',') ]
        if items[0] == 'cmd': # exécuter une commande
            hide, nowait = False, False
            if 'HIDDEN' in items: hide = True
            if 'NOWAIT' in items: nowait = True
            return self.gen_cmd_line(items[1], hide, nowait)
        elif items[0] == 'lecteur': # monter un partage
            lettre, partage = items[1], items[2]
            return self.gen_lecteur_bloc(lettre, partage)
        else: return ''

    def write_fich(self):
        """Ecrit le fichier
        """
        try:
            #bat_file = '/home/netlogon/scripts/os/%s/%s.bat'%(os_type, user)
            if self.os_type == 'Win95':
                bat_file = '/home/netlogon/%s%s.bat'%(self.user, self.os_type)
            else:
                bat_file = '/home/netlogon/%s%s.txt'%(self.user, self.os_type)
            fic = file(bat_file, 'w')
            fic.writelines(self.debut_script)
            fic.writelines(self.milieu_script)
            fic.writelines(self.fin_script)
            fic.close()
    
            os.system('/usr/bin/todos -u %s'%bat_file)
            # Faut-il vraiment que le fichier appartienne à 'user' ? (possible faille ?)
            #os.system('chown %s %s'%(self.user, bat_file))
            os.system('chmod 644 %s'%bat_file)
            
        except:
            sys.exit("erreur d'écriture du fichier %s"%bat_file)

    #########
    # Win9x #
    #########
    def gest_win95(self):
        """Gestion "à l'ancienne" pour Win9x/Me
        exécuté par le service et logon.py(.exe) sur les clients
        """
        contenu = self.gen_vnc_line()
        contenu += self.gen_esu()
        return contenu

    def test_esu(self):
        for i in os.listdir('/home/esu/Base'):
            if i.lower() == 'esuclnt.exe':esuline = '\\\\%s\\ESU\\Base\\esuclnt.exe'%self.smb_serveur
            return True
        return False

    def gen_esu(self):
        """Ajoute la ligne pour Esu
        """
        if self.test_esu():
            esuline = '\\\\%s\\ESU\\Base\\esuclnt.exe'%self.smb_serveur
            return self.gen_cmd_line(esuline, hide=True)
        return ''

    def gen_vnc_line(self):
        """VNC utilise maintenant login/pass du domaine
        ce .reg spécifie le type de controle possible (desactive, simple, controle) lors de l'observation pour les Win95
        """
        vnccontrole = 'regedit /s \\\\%s\\netlogon\\scripts\\os\\Win95\\vnc_controle.reg'%self.smb_serveur
        vnckill = '%windir%\\eole\\ultravnc\\winvnc -kill'
        vncservice = '%windir%\\eole\\ultravnc\\winvnc -service'
        if 'professeurs' in self.groups:
            bloc = self.gen_cmd_line(vnccontrole)
            bloc += self.gen_cmd_line(vnckill)
        if 'eleves' in self.groups:
            bloc = self.gen_cmd_line(vnccontrole)
            bloc += self.gen_cmd_line(vnckill)
            bloc += self.gen_cmd_line(vncservice)
        return bloc

#############################################################
## Boucle principale du programme - génération des fichiers
#############################################################
def mhelp():
    print ("""
    Utilisation: %s

    -u --user <nom de login>
    -o --os <type os>
    -m --machine <nom de machine>
    -i --ip <adresse ip>
    """%os.path.basename(sys.argv[0]))

def main():
    # initialisation des données
    user = None
    ostype = None
    machine = None
    groups = None
    adresse_ip = None
    
    # on récupère le nom du serveur samba
    Dico=parsedico.parse_dico()
    try:
        smb_serveur=Dico['smb_netbios_name']
    except:
        sys.exit("Erreur, le nom de ce serveur n'est pas défini dans la configuration Eole")
    
    # on lit l'utilisateur passé en variable
    try:
        options, args = getopt.getopt(sys.argv[1:],'u:o:m:i:h',
                ['user=',
                'os=',
                'machine=',
                'ip=',
                'help'])

    except getopt.GetoptError:
        er_type, er_value, er_traceback = sys.exc_info()
        er_msg = str(er_value)
        print "L'option %s est inconnue ou nécessite un argument" % er_msg.split(' ')[1]
        mhelp()
        sys.exit(1)

            
    for (opt,val) in options:
        # uid de l'utilisateur
        if (opt == '-u') or (opt == '--user'):
            user = val
            user = user.strip()
            user = user.replace(' ','')
            if not user[0].isalnum(): user = None
        
        # système d'exploitation
        elif (opt == '-o') or (opt == '--os'):
            ostype = val
            
        # nom de la machine
        elif (opt == '-m') or (opt == '--machine'):
            machine = val
            
        # adresse ip client
        elif (opt == '-i') or (opt == '--ip'):
            adresse_ip = val
        elif (opt == '-h') or (opt == '--help'):
            mhelp()
            sys.exit(0)
        else:
            mhelp()
            sys.exit(1)
	    
    if (not user or not ostype or not machine or not adresse_ip):
        print "Il manque des arguments"
        mhelp()
        sys.exit(1)
    
    if (user=="nobody"):
        sys.exit("utilisateur nobody : ne rien faire")
    
    # construction du script de connexion
    # la variable "serveur" est dans "from scribe.backend_conf import *"
    logon(serveur).build_netlogon(user, ostype, machine, adresse_ip)

if __name__ == '__main__':
    main()
