#!/usr/bin/env python
# -*- coding: utf-8 -*-

#########################################################
# utilitaire de gestion des groupes spéciaux
# licence GPL Eole
# Bruno Boiget <bruno.boiget@ac-dijon.fr>
# Joël Cuissinat <joel.cuissinat@ac-dijon.fr>
#########################################################

###################
# modules importés
from scribe import backend_conf
#from os import system,sep,unlink
import os, ldap, sys, getopt
import special_shares as shares
#####################




class specialShares:
    def __init__(self):
        self.share_template = """# -*- coding: utf-8 -*-
# nom des partages présentant des cas particuliers (lettre de lecteur réservée)
# lettres réservées Eole
eole_shares = %s
# lettres réservées utilisateurs
special_shares = %s
        """
        self.sp_sharef = os.path.join(backend_conf.tool_path, 'special_shares.py')
        self.sp_sharef_pyc = os.path.join(backend_conf.tool_path, 'special_shares.pyc')

    def flush_spe_shares(self):
        try:
            os.unlink(self.sp_sharef_pyc)
            fh = file(self.sp_sharef, "w")
            fh.write(self.share_template%(shares.eole_shares, shares.special_shares))
            fh.close()
        except:
            return False
        return True

    def test_drive(self, drive):
        if not drive.endswith(":"):
            if len(drive) > 1:
                print 'Lettre de lecteur non valide "%s"'%drive
                sys.exit()
            drive += ':'
        else:
            if len(drive) > 2:
                print 'Lettre de lecteur non valide "%s"'%drive
                sys.exit()
        return drive.upper()

    def verif_drive(self, drive):
        drive = self.test_drive(drive)
        if drive in shares.eole_shares.values() or drive == 'U:':
            print'"%s" est une lettre Eole réservée'%drive
            sys.exit()

    def verif_share(self, share):
        if share in shares.eole_shares.keys():
            print '"%s" est un partage Eole non modifiable'%share
            sys.exit()

    def add_special_share(self, sharename, drive):
        """Fonction d'ajout/modification d'un partage spécial
        """
        reload(shares)
        drive = self.test_drive(drive)
        self.verif_drive(drive)
        self.verif_share(sharename)
        shares.special_shares[sharename] = drive
        print 'Réservation de la lettre de lecteur "%s" pour le partage "%s"'%(drive, sharename)
        return self.flush_spe_shares()
    
    def del_special_share(self, sharename):
        """Fonction de suppression de la lettre réservée d'un partage spécial
        """
        reload(shares)
        self.verif_share(sharename)
        if shares.special_shares.has_key(sharename):
            print 'Désactivation de la réservation de lettre pour le partage "%s"'%sharename
            del(shares.special_shares[sharename])
        else:
            print """Aucune lettre n'était associé au partage "%s" """%sharename
            sys.exit()
        return self.flush_spe_shares()
    
    def get_letter(self, sharename):
        """Fonction retournant la lettre associée à un partage spécial (ou "" si pas de lettre)
        """
        reload(shares)
        lettre_dict = self.get_shares_dict()
        if lettre_dict.has_key(sharename):
            return '%s %s'%(lettre_dict[sharename], sharename)
        return ''
    
    def get_share(self, drive):
        """Retourne le partage associé à "drive" si existant, "" sinon
        """
        lettre_dict = self.get_shares_dict()
        drive = self.test_drive(drive)
        for share in lettre_dict.keys():
            if lettre_dict[share] == drive: 
                return '%s %s'%(drive, share)
        return ''

    def get_shares_dict(self):
        reload(shares)
        lettre_dict = shares.special_shares
        lettre_dict.update(shares.eole_shares)
        return lettre_dict

    def get_shares(self):
        """Fonction de lecture des partages reliés à un *groupe*
        """
        try:
            reload(backend_conf)
            query=ldap.open(backend_conf.serveur)
            query.simple_bind_s("cn=admin, o=gouv, c=fr", backend_conf.ldap_passwd)
            result_shares=query.search_s("o=gouv, c=fr", ldap.SCOPE_SUBTREE, \
            "(objectClass=sambaFileShare)")
            query.unbind()
        except:
            print 'Erreur connexion LDAP, Abandon !'
            sys.exit()

        dico_shares={}
        for share in result_shares:
            dico_shares[share[1]['sambaShareName'][0]]=share[1]['sambaShareGroup'][0]
        return dico_shares
    
    
    def print_shares(self):
        """Fonction d'affichage des partages (pour EAD par exemple)
        """
        # récupération des partages, classement alphabétique
        shares_list = self.get_shares().keys()
        shares_list.sort()
        # parcours des partages et affichage Partage+(Lettre)+Groupe
        # SCRIBE : on n'affiche que les partages spéciaux !
        reload(shares)
        for share in shares_list:
            if shares.eole_shares.has_key(share):
                lettre = shares.eole_shares[share]
                print '%s %s (réservée Eole)'%(lettre, share)
            if shares.special_shares.has_key(share):
                lettre = shares.special_shares[share]
                print '%s %s'%(lettre, share)
        print 'U: perso (réservée Eole)'

    def get_reserved_letters(self):
        """Fonction retournant les partages pour backend Ead2
        """
        d = {}
        shares_list = self.get_shares().keys()
        reload(shares)
        for share in shares_list:
            if shares.eole_shares.has_key(share):
                lettre = shares.eole_shares[share]
                d[share] = lettre
            elif shares.special_shares.has_key(share):
                lettre = shares.special_shares[share]
                d[share] = lettre
            else:
                d[share] = ''
        d['perso'] = 'U:'
        return d

# si appelé en ligne de commande...
def main():
    # définition du message d'aide
    help_msg ="""arguments:
         -p (--partage) nom du partage
         -l (--lettre) lettre sur laquelle connecter le partage sous windows
         -h (--help) (ce message)

     exemples : 
     
     * ajout/modification de la lettre de partage      : %(app)s -p partage -l J:
     * suppression de la lettre réservée               : %(app)s -p partage -l ""
     * récupération de la lettre associée à un partage : %(app)s -p partage
     * récupération du partage associé à une lettre    : %(app)s -l J:
     * récupération de la liste des partage spéciaux   : %(app)s
     
     """%dict(app = os.path.basename(sys.argv[0]))

    # récupération des arguments de la ligne de commande
    try:
        options, args = getopt.getopt(sys.argv[1:],'p:l:h',
                                 ['partage=',
                                  'lettre=',
                                  'help'])
    except getopt.GetoptError:
        # on récupère les information de l'exception
        er_type, er_value, er_traceback = sys.exc_info()
        er_msg = str(er_value)
        # on prévient l'utilisateur que l'option nécessite un argument
        sys.exit("l'option -%s est inconnue ou nécessite un argument" % er_msg[er_msg.index('-')+1])

    partage = None
    letter = None
    sp_share = specialShares()

    for (opt,val) in options:
        if (opt == '-p') or (opt == '--partage'):
            partage = val

        elif (opt == '-l') or (opt == '--lettre'):
            letter = val

        elif (opt == '-h') or (opt == '--help'):
            sys.exit(help_msg)

    # sans arguments => affichage
    if partage is None:
        if letter is None:
            sys.exit(sp_share.print_shares())
        else:
            sys.exit(sp_share.get_share(letter))
    else:
        if letter is None:
            sys.exit(sp_share.get_letter(partage))
        elif letter == '':
            sys.exit(sp_share.del_special_share(partage))
        else:
            sys.exit(sp_share.add_special_share(partage, letter))

if __name__ == '__main__':
    main()
