#!/usr/bin/env python
# -*- coding: UTF-8 -*-

###############################################################
# utilitaire de suppression d'un groupe dans l'annuaire d'Horus
# licence GPL Eole
# conception par Bruno Boiget <bruno.boiget@ac-dijon.fr>
#########################################################

#import des modules nécessaires
import sys
import ldap
import manage_special_shares
from os import system
from scribe.backend_conf import *

#####################
# variables globales
#####################

passwd = ldap_passwd

##################
# fonctions LDAP
##################

def get_shares(groupe,serveur):
    """Fonction de lecture des partages reliés à un groupe
    """
    try:
        query=ldap.open(serveur)
        query.simple_bind_s("cn=admin, o=gouv, c=fr",passwd)
        result_shares=query.search_s("o=gouv, c=fr", ldap.SCOPE_SUBTREE, \
        "(&(objectClass=sambaFileShare)(sambaShareGroup="+groupe+"))")
        query.unbind()
    except:
        return "Erreur de lecture LDAP"
    else:
        liste_shares=[]
        for share in result_shares:
            liste_shares.append(share[1]['sambaShareName'][0])
        return liste_shares

# supression d'un partage dans l'annuaire
def del_share(share_name):
    """fonction de supression d'un partage dans l'annuaire LDAP
    """
    from special_shares import special_shares
    try:
        query=ldap.open(serveur)
        query.simple_bind_s("cn=admin, o=gouv, c=fr",passwd)
        # on récupère le dn
        result_share=query.search_s("o=gouv, c=fr", ldap.SCOPE_SUBTREE, \
        "(&(objectClass=sambaFileShare)(sambaShareName="+share_name+"))")
        # on supprime l'entrée de l'annuaire
        query.delete(result_share[0][0])
        query.unbind()
    except:
        return "Erreur de supression dans LDAP"
    else:
        a = manage_special_shares.specialShares()
        a.del_special_share(share_name)
        return "OK"


def del_group(groupe,ignore_shares = 0):
#    try:
    query=ldap.open(serveur)
    query.simple_bind_s("cn=admin, o=gouv, c=fr",passwd)
    try:
        result_group=query.search_s("o=gouv, c=fr", ldap.SCOPE_SUBTREE, \
        "(&(objectClass=posixGroup)(cn="+groupe+"))")
    except:
        query.unbind()
        return "erreur de lecture des groupes dans ldap",query.get_option(ldap.OPT_HOST_NAME)
    else:
        if result_group == []:
            return "erreur le groupe "+groupe+" n'existe pas"
        # on regarde si il reste des utilisateurs inscrits à ce groupe
        try:
            users = result_group[0][1]['memberUid']
        except KeyError:
            # pas d'utilisateurs, on continue
            # on regarde si il existe des partages pour ce groupe
            list_shares = get_shares(groupe,serveur)
            if list_shares == []:
                try:
                    # supression effective du groupe
                    query.delete(result_group[0][0])
                    query.unbind()
                except:
                    return "erreur lors de la supression de "+groupe
                else:
                    return "groupe "+groupe+" supprimé"
            else:
                # on a décidé de supprimer les partages 
                if ignore_shares == 1:
                    for share in list_shares:
                        del_share(share)
                    try:
                        # supression effective du groupe
                        query.delete(result_group[0][0])
                        query.unbind()
                    except:
                        return "erreur lors de la supression de "+groupe
                    else:
                        return "groupe "+groupe+" supprimé"
                else:
                # erreur, le groupe est encore lié à des partages
                    query.unbind()
                    return "Erreur, le groupe "+groupe+" est lié au(x) partage(s) "+str(list_shares)
        else:
            query.unbind()
            # il reste des utilisateurs dans le groupe
            return "Erreur, le groupe "+groupe+" contient encore des utilisateurs"

            
#####################
# programme principal
#####################

def main():
    ignore_shares=0
    try:
        # on affiche le message d'aide si besoin
        if sys.argv[1] in ["--help","-h"]:
            print ("\n syntaxe : "+sys.argv[0]+" groupe1,groupe2,...,groupeN \n         ou "+sys.argv[0]+" pour le mode interactif \n")
            return
        # on regarde si un groupe est passé en ligne de commande
        groupes = sys.argv[1].split(',')
    except:
        # demande du groupe à supprimer
        groupes = []
        groupe = ""
        fin_liste = 0
        while fin_liste == 0:
            groupe = raw_input("entrez un nom de groupe (rien pour terminer la liste) :")
            if groupe != "":
                groupes.append(groupe)
            else:
                fin_liste = 1
                
        rep = "bidon"
        while (rep.upper() != 'O') and (rep.upper() != 'N'):
            rep = 'N'
            rep = raw_input("voulez vous supprimer de l'annuaire les partages associés au groupe ? (N) :")
            
        if rep.upper() == 'O':
            ignore_shares = 1

    # supression des groupes demandés
    for groupe in groupes:
        if ignore_shares == 1:
            print del_group(groupe,ignore_shares=1)
        else:
            print del_group(groupe)


if __name__ == '__main__':
    main()
